package main

import (
	"database/sql"
	"encoding/json"
	"intimate"
	"log"
	"os"
	"os/signal"
	"regexp"
	"sync/atomic"
	"syscall"
	"testing"
	"time"

	"github.com/tebeka/selenium"
)

// sstore  源存储实例, 为存储源数据的实现. 表格具体参考sql/intimate_source.sql
var sstore *intimate.StoreSource = intimate.NewStoreSource(string(intimate.STTwitch))

// estore 解析存储连接实例
var estore *intimate.StoreExtractor = intimate.NewStoreExtractor()

func TestCase(t *testing.T) {
	var loop int32 = 1
	wd := intimate.GetChromeDriver(3030)

	go func() {
		signalchan := make(chan os.Signal)
		signal.Notify(signalchan, syscall.SIGKILL, syscall.SIGQUIT, syscall.SIGTERM, syscall.SIGSTOP)
		log.Println("accept stop command:", <-signalchan)
		atomic.StoreInt32(&loop, 0)
	}()

	var lasterr error = nil
	// var err error

	for atomic.LoadInt32(&loop) > 0 {
		streamer, err := estore.Pop(intimate.Ptwitch, 0)
		if streamer == nil || err != nil {
			if err != lasterr {
				log.Println(err, lasterr)
				lasterr = err
			}
			time.Sleep(time.Second * 2)
			continue
		}

		var updateUrl map[string]string
		json.Unmarshal(streamer.UpdateUrl.([]byte), &updateUrl)
		liveUrl := updateUrl["live"]
		log.Println(liveUrl)

		// err = wd.Get("https://www.twitch.tv/zoe_0601" + "/about")
		err = wd.Get(liveUrl + "/about")
		if err != nil {
			log.Println(err)
			//estore.UpdateError(streamer, err)
			continue
		}

		streamer.LiveUrl = sql.NullString{String: liveUrl, Valid: true}
		clog := &intimate.CollectLog{}
		clog.UserId = streamer.UserId

		time.Sleep(time.Millisecond * 500)
		extractUserName(wd, streamer)
		extractFollowers(wd, clog)
		err = extractViews(wd, clog) // views + tags + gratuity

		if err != nil {
			// 不直播时提取礼物 gratuity
			wd.WaitWithTimeout(func(web selenium.WebDriver) (bool, error) {
				channelchat, err := wd.FindElement(selenium.ByXPATH, `//a[@data-a-target="channel-home-tab-Chat"]`)
				btn, _ := web.FindElement(selenium.ByXPATH, `//button[@data-test-selector="expand-grabber"]`)
				if (err == nil && channelchat != nil) || btn != nil {
					if channelchat != nil {
						channelchat.Click()
					}
					time.Sleep(time.Second)
					extractGratuity(wd, clog)
					return true, nil
				}
				return false, nil

			}, time.Second*4)
		}

		streamer.Platform = intimate.Ptwitch
		clog.Platform = string(streamer.Platform)
		clog.UpdateTime = sql.NullTime{Time: time.Now(), Valid: true}
		lastClogId := estore.InsertCollectLog(clog)

		streamer.Operator = 100
		streamer.LatestLogUid = lastClogId
		estore.UpdateStreamer(streamer)

	}
}

func extractUserName(wd selenium.WebDriver, streamer *intimate.Streamer) error {
	return wd.WaitWithTimeout(func(web selenium.WebDriver) (bool, error) {
		label, err := web.FindElement(selenium.ByXPATH, "//a[@class='tw-interactive']//h1")
		if err == nil {
			if ltxt, err := label.Text(); err == nil {
				log.Println("label:", ltxt)
				streamer.UserName = sql.NullString{String: ltxt, Valid: true}
				return true, nil
			}
		}
		return false, err
	}, 6*time.Second)
}

func extractFollowers(wd selenium.WebDriver, clog *intimate.CollectLog) error {
	return wd.WaitWithTimeout(func(web selenium.WebDriver) (bool, error) {
		efollowers, err := web.FindElement(selenium.ByXPATH, "//div[@data-a-target='about-panel']//div[@class='tw-align-center']")
		if err != nil {
			return false, err
		}
		followers, err := efollowers.Text()
		if err != nil || followers == "" {
			return false, err
		}
		followers = regexp.MustCompile(`[\d,]+`).FindString(followers)
		fint, _ := intimate.ParseNumber(followers)
		clog.Followers = sql.NullInt64{Int64: int64(fint), Valid: true}
		log.Println("followers: ", followers, fint)
		return true, nil
	}, 6*time.Second)
}

func extractViews(wd selenium.WebDriver, clog *intimate.CollectLog) error {
	return wd.WaitWithTimeout(func(web selenium.WebDriver) (bool, error) {
		views, err := web.FindElement(selenium.ByXPATH, "//a[@data-a-target='home-live-overlay-button']/span")
		if views != nil {
			if txt, err := views.Text(); err == nil {

				vint, _ := intimate.ParseNumber(txt)
				clog.Views = sql.NullInt64{Int64: vint, Valid: true}
				log.Println("views:", txt)
				views.Click()

				extractTags(wd, clog)
				extractTitle(wd, clog)
				extractGratuity(wd, clog)

				return true, nil
			}
		}
		return false, err
	}, time.Second*4)
}

func extractTitle(wd selenium.WebDriver, clog *intimate.CollectLog) error {
	return wd.WaitWithTimeout(func(web selenium.WebDriver) (bool, error) {
		title, err := web.FindElement(selenium.ByXPATH, `//h2[@data-a-target='stream-title']`)
		if err == nil {
			if txt, err := title.Text(); err == nil {
				clog.LiveTitle = sql.NullString{String: txt, Valid: true}
				return true, nil
			}
		}
		return false, err
	}, time.Second*4)
}

func extractTags(wd selenium.WebDriver, clog *intimate.CollectLog) error {
	return wd.WaitWithTimeout(func(web selenium.WebDriver) (bool, error) {
		tags, err := web.FindElements(selenium.ByXPATH, "//a[@aria-label and @data-a-target and @href]/div[@class and text()]")
		if len(tags) == 0 {
			return false, err
		}

		var stags []string
		for _, tag := range tags {
			if txt, err := tag.Text(); err == nil {
				stags = append(stags, txt)
			} else {
				log.Println(err)
			}
			log.Println(tag.Text())
		}
		if len(stags) > 0 {
			if tagbuf, err := json.Marshal(stags); err == nil {
				clog.Tags = tagbuf
			} else {
				log.Println(err)
			}
		}

		return true, nil
	}, time.Second*4)
}

func extractGratuity(wd selenium.WebDriver, clog *intimate.CollectLog) error {
	return wd.WaitWithTimeout(func(web selenium.WebDriver) (bool, error) {
		btn, err := web.FindElement(selenium.ByXPATH, `//button[@data-test-selector="expand-grabber"]`)
		if err == nil {
			btn.Click()
			time.Sleep(time.Second)
			gifcount, err := web.FindElements(selenium.ByXPATH, `//div[@class="sub-gift-count tw-flex"]/p`)
			if err == nil {
				var gratuity int64 = 0
				for _, gc := range gifcount {
					if gtxt, err := gc.Text(); err == nil {
						gint, _ := intimate.ParseNumber(gtxt)
						gratuity += gint
					} else {
						log.Println(err)
					}
				}
				clog.Gratuity = sql.NullInt64{Int64: gratuity, Valid: true}
			}
			return true, nil
		}

		return false, err
	}, time.Second*4)
}
